Package: methylscaper
Type: Package
Title: Visualization of Methylation Data
Description: methylscaper is an R package for processing and visualizing data jointly
  profiling methylation and chromatin accessibility (MAPit, NOMe-seq, scNMT-seq, nanoNOMe,
  etc.). The package supports both single-cell and single-molecule data, and 
  a common interface for jointly visualizing both data types through the generation of
  ordered representational methylation-state matrices. The Shiny app allows for an 
  interactive seriation process of refinement and re-weighting that optimally 
  orders the cells or DNA molecules to discover methylation patterns and nucleosome 
  positioning.
Version: 1.21.0
Authors@R: c(person("Bacher", "Rhonda", email = "rbacher@ufl.edu", role = c("aut", "cre")), person("Parker", "Knight", role = c("aut")))
Depends: R (>= 4.4.0)
License: GPL-2
Encoding: UTF-8
Imports: shiny, shinyjs, seriation, BiocParallel, seqinr, Biostrings,
        pwalign, Rfast, grDevices, graphics, stats, utils, tools,
        methods, shinyFiles, data.table, SummarizedExperiment
RoxygenNote: 7.3.2
Suggests: BiocStyle, knitr, rmarkdown, devtools, R.utils
VignetteBuilder: knitr
biocViews: DNAMethylation, Epigenetics, Sequencing, Visualization,
        SingleCell, NucleosomePositioning
URL: https://github.com/rhondabacher/methylscaper/
BugReports: https://github.com/rhondabacher/methylscaper/issues
git_url: https://git.bioconductor.org/packages/methylscaper
git_branch: devel
git_last_commit: 013af48
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-29
NeedsCompilation: no
Packaged: 2026-04-29 22:53:10 UTC; biocbuild
Author: Bacher Rhonda [aut, cre],
  Parker Knight [aut]
Maintainer: Bacher Rhonda <rbacher@ufl.edu>
